@@ -30,6 +30,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
+ *	from: @(#)lp.h	5.5 (Berkeley) 6/1/90
+ *	$Id: lp.h,v 1.3 1993/12/08 00:47:03 jtc Exp $
  */
 
 /*
@@ -45,16 +47,13 @@
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#include <grp.h>
 #include <pwd.h>
 #include <syslog.h>
 #include <signal.h>
 #include <sys/wait.h>
 #include <sgtty.h>
-#include  <termios.h>
 #include <ctype.h>
 #include <errno.h>
-#include <string.h>
 #include "lp.local.h"
 
 extern int	DU;		/* daeomon user-id */
@@ -89,11 +88,8 @@ extern short	PX;		/* page width in pixels */
 extern short	PY;		/* page length in pixels */
 extern short	PL;		/* page length */
 extern short	BR;		/* baud rate if lp is a tty */
-/* Changes to allow serial printer to work with hardware handshake*/
-extern ulong	FC;		/* flags to clear if lp is a tty */
-extern ulong	FS;		/* flags to set if lp is a tty */
-extern ulong	XC;		/* flags to clear for local mode */
-extern ulong	XS;    		/* flags to set for local mode */
+extern int	FC;		/* flags to clear if lp is a tty */
+extern int	FS;		/* flags to set if lp is a tty */
 extern short	RS;		/* restricted to those with local accounts */
 
 extern char	line[BUFSIZ];
@@ -101,7 +97,7 @@ extern char	pbuf[];		/* buffer for printcap entry */
 extern char	*bp;		/* pointer into ebuf for pgetent() */
 extern char	*name;		/* program name */
 extern char	*printer;	/* printer name */
-extern char	host[MAXHOSTNAMELEN];	/* host machine name */
+extern char	host[32];	/* host machine name */
 extern char	*from;		/* client's machine name */
 extern int	sendtorem;	/* are we sending to a remote? */
 extern int	errno;
