@@ -31,16 +31,16 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)pathnames.h	5.4 (Berkeley) 6/1/90
- *	$Id: pathnames.h,v 1.4 1999/08/27 22:37:19 notting Exp $
+ *	$Id: pathnames.h,v 1.2 1993/08/01 17:59:06 mycroft Exp $
  */
 
 #include <paths.h>
 
-#define	_PATH_DEFDEVLP		"/dev/lp0"
+#define	_PATH_DEFDEVLP		"/dev/lp"
 #define	_PATH_DEFSPOOL		"/var/spool/lpd"
 #define	_PATH_HOSTSEQUIV	"/etc/hosts.equiv"
 #define	_PATH_HOSTSLPD		"/etc/hosts.lpd"
-#define	_PATH_MASTERLOCK	"/var/spool/lpd/lpd.lock"
+#define	_PATH_MASTERLOCK	"/var/run/lpd.pid"
 #define	_PATH_PR		"/usr/bin/pr"
 #define	_PATH_PRINTCAP		"/etc/printcap"
 #define	_PATH_SOCKETNAME	"/dev/printer"
