@@ -31,6 +31,11 @@
  * SUCH DAMAGE.
  */
 
+#ifndef lint
+/*static char sccsid[] = "from: @(#)printjob.c	5.13 (Berkeley) 3/2/91";*/
+static char rcsid[] = "$Id: printjob.c,v 1.2 1993/08/01 17:58:51 mycroft Exp $";
+#endif /* not lint */
+
 /*
  * printjob -- print jobs in the queue.
  *
@@ -41,9 +46,6 @@
 #include "lp.h"
 #include "pathnames.h"
 
-
-#include <string.h>
-
 #define DORETURN	0	/* absorb fork error */
 #define DOABORT		1	/* abort if dofork fails */
 
@@ -57,68 +59,43 @@
 #define	NOACCT		2
 #define	FILTERERR	3
 #define	ACCESS		4
-#define UNLINK		5
 
-static dev_t	 fdev;		/* device of file pointed to by symlink */
-static ino_t	 fino;		/* inode of file pointed to by symlink */
-static FILE	*cfp;		/* control file */
-static int	 child;		/* id of any filters */
-static int	 lfd;		/* lock file descriptor */
-static int	 ofd;		/* output filter file descriptor */
-static int	 ofilter;	/* id of output filter, if any */
+char	title[80];		/* ``pr'' title */
+FILE	*cfp;			/* control file */
+int	pfd;			/* printer file descriptor */
+int	ofd;			/* output filter file descriptor */
+int	lfd;			/* lock file descriptor */
+int	pid;			/* pid of lpd process */
+int	prchild;		/* id of pr process */
+int	child;			/* id of any filters */
+int	ofilter;		/* id of output filter, if any */
+int	tof;			/* true if at top of form */
+int	remote;			/* true if sending files to remote */
+dev_t	fdev;			/* device of file pointed to by symlink */
+ino_t	fino;			/* inode of file pointed to by symlink */
 
-static int	 pfd;		/* printer file descriptor */
-static int	 pid;		/* pid of lpd process */
-static int	 prchild;	/* id of pr process */
-static char	 title[80];	/* ``pr'' title */
-static int	 tof;		/* true if at top of form */
+char	fromhost[32];		/* user's host machine */
+char	logname[32];		/* user's login name */
+char	jobname[100];		/* job or file name */
+char	class[32];		/* classification field */
+char	width[10] = "-w";	/* page width in characters */
+char	length[10] = "-l";	/* page length in lines */
+char	pxwidth[10] = "-x";	/* page width in pixels */
+char	pxlength[10] = "-y";	/* page length in pixels */
+char	indent[10] = "-i0";	/* indentation size in characters */
+char	tempfile[] = "errsXXXXXX"; /* file name for filter output */
 
-static char	class[32];		/* classification field */
-static char	fromhost[MAXHOSTNAMELEN];		/* user's host machine */
-				/* indentation size in static characters */
-static char	indent[10] = "-i0";
-static char	jobname[100];		/* job or file name */
-static char	extparms[256];		/* extended parms */
-static char	length[10] = "-l";	/* page length in lines */
-static char	logname[32];		/* user's login name */
-static char	euser[32];		/* effective user */
-static char	pxlength[10] = "-y";	/* page length in pixels */
-static char	pxwidth[10] = "-x";	/* page width in pixels */
-static char	tempfile[] = "errsXXXXXX"; /* file name for filter errors */
-static char	width[10] = "-w";	/* page width in static characters */
+static short SF;		/* suppress FF on each print job */
 
-	int	remote;		/* true if sending files to remote */
-static	short SF;		/* suppress FF on each print job */
-
-static void	abortpr	(int signo);
-static void	alarmhandler	(int signo);
-static void	banner	(char *name1, char *name2);
-static int	dofork	(int action);
-static int	doforkuser(int action, char *user);
-static int	dropit	(int c);
-static void	init	(void);
-static void	openpr	(void);
-static int	print	(int format, char *file);
-static int	printit	(char *file);
-static void	pstatus	(const char *msg, ...);
-static char	response(void);
-static void	scan_out(int scfd, char *scsp, int dlm);
-static char *	scnline	(int key, char *p, int c);
-static int	sendit	(char *file);
-static int	sendfile(int type, char *file);
-static void	sendmail(char *user, int bombed);
-static void	setty	(void);
-static void	unlinkfile(char *file, char *host, char *user);
-
-void
 printjob()
 {
 	struct stat stb;
 	register struct queue *q, **qp;
 	struct queue **queue;
 	register int i, nitems;
-	off_t pidoff;
-	int errcnt, count = 0;
+	long pidoff;
+	int count = 0;
+	void abortpr();
 
 	init();					/* set up capabilities */
 	(void) write(1, "", 1);			/* ack that daemon is started */
@@ -161,7 +138,7 @@ printjob()
 	/*
 	 * write process id for others to know
 	 */
-	snprintf(line, BUFSIZ, "%u\n", pid);
+	sprintf(line, "%u\n", pid);
 	pidoff = i = strlen(line);
 	if (write(lfd, line, i) != i) {
 		syslog(LOG_ERR, "%s: %s: %m", printer, LO);
@@ -191,10 +168,9 @@ again:
 		q = *qp++;
 		if (stat(q->q_name, &stb) < 0)
 			continue;
-		errcnt = 0;
 	restart:
 		(void) lseek(lfd, pidoff, 0);
-		(void) snprintf(line, sizeof(line), "%s\n", q->q_name);
+		(void) sprintf(line, "%s\n", q->q_name);
 		i = strlen(line);
 		if (write(lfd, line, i) != i)
 			syslog(LOG_ERR, "%s: %s: %m", printer, LO);
@@ -222,13 +198,12 @@ again:
 		}
 		if (i == OK)		/* file ok and printed */
 			count++;
-		else if (i == REPRINT && ++errcnt < 5) {
-			/* try reprinting the job */
+		else if (i == REPRINT) { /* try reprinting the job */
 			syslog(LOG_INFO, "restarting %s", printer);
 			if (ofilter > 0) {
 				kill(ofilter, SIGCONT);	/* to be sure */
 				(void) close(ofd);
-				while ((i = wait(NULL)) > 0 && i != ofilter)
+				while ((i = wait(0)) > 0 && i != ofilter)
 					;
 				ofilter = 0;
 			}
@@ -237,17 +212,6 @@ again:
 				syslog(LOG_WARNING, "%s: %s: %m", printer, LO);
 			openpr();		/* try to reopen printer */
 			goto restart;
-		} else {
-			syslog(LOG_WARNING, "%s: job could not be %s (%s)", printer,
-				remote ? "sent to remote host" : "printed", q->q_name);
-			if (i == REPRINT) {
-				/* ensure we don't attempt this job again */
-				(void) unlink(q->q_name);
-				q->q_name[0] = 'd';
-				(void) unlink(q->q_name);
-				if (logname[0])
-					sendmail(logname, FATALERR);
-			}
 		}
 	}
 	free((char *) queue);
@@ -266,8 +230,6 @@ again:
 			if (TR != NULL)		/* output trailer */
 				(void) write(ofd, TR, strlen(TR));
 		}
-		(void) close(ofd);
-		(void) wait(NULL);
 		(void) unlink(tempfile);
 		exit(0);
 	}
@@ -287,7 +249,6 @@ char ifonts[4][40] = {
  * The remaining part is the reading of the control file (cf)
  * and performing the various actions.
  */
-static int
 printit(file)
 	char *file;
 {
@@ -325,7 +286,6 @@ printit(file)
 	 *		H -- "host name" of machine where lpr was done
 	 *              P -- "person" user's login name
 	 *              I -- "indent" amount to indent output
-	 *		R -- laser dpi "resolution"
 	 *              f -- "file name" name of text file to print
 	 *		l -- "file name" text file with control chars
 	 *		p -- "file name" text file to print with pr(1)
@@ -347,55 +307,24 @@ printit(file)
 	 *      getline reads a line and expands tabs to blanks
 	 */
 
-	/* pass 0 */
-    /* Henri Gomez <gomez@slib.fr> 19991104 */
-	/* Pre scan to get the Z flags (RFC Filter Extension) */
-	while (getline(cfp))
-		switch (line[0]) {
-        case 'Z':
-            if (line[1] != '\0') {
-                strncpy(extparms, line+1, sizeof(extparms) - 1);
-                extparms[sizeof(extparms) - 1] = '\0';
-            } else
-                strcpy(extparms, " ");
-            continue;
-	}
-
-	rewind(cfp);
-
 	/* pass 1 */
 
 	while (getline(cfp))
 		switch (line[0]) {
 		case 'H':
-			strncpy(fromhost, line+1, sizeof(fromhost) - 1);
-			fromhost[sizeof(fromhost) - 1] = '\0';
-			if (class[0] == '\0') {
-				strncpy(class, line+1, sizeof(class) - 1);
-				class[sizeof(class) - 1] = '\0';
-			}
+			strcpy(fromhost, line+1);
+			if (class[0] == '\0')
+				strncpy(class, line+1, sizeof(class)-1);
 			continue;
 
 		case 'P':
-			{
-			struct passwd *dude;
-
-			strncpy(logname, line+1, sizeof(logname) - 1);
-			logname[sizeof(logname) - 1] = '\0';
-			dude = getpwnam(logname);
-
-			if (RS && dude == NULL) {	/* no such user plus
-							 * restricted */
+			strncpy(logname, line+1, sizeof(logname)-1);
+			if (RS) {			/* restricted */
+				if (getpwnam(logname) == (struct passwd *)0) {
 					bombed = NOACCT;
 					sendmail(line+1, bombed);
 					goto pass2;
-			}
-			if(dude == NULL || dude->pw_uid == 0)
-				   /* if user root, set euid to lp  */
-				strncpy(euser, "lp", 3);
-			else
-				   /* since the user does exist run filter as user */
-				strncpy(euser, logname, sizeof(euser));
+				}
 			}
 			continue;
 
@@ -412,28 +341,22 @@ printit(file)
 			fino = i;
 			continue;
 
-		case 'Z':
-			continue; /* Allready handled */
-
 		case 'J':
-			if (line[1] != '\0') {
-				strncpy(jobname, line+1, sizeof(jobname) - 1);
-				jobname[sizeof(jobname) - 1] = '\0';
-			} else
+			if (line[1] != '\0')
+				strncpy(jobname, line+1, sizeof(jobname)-1);
+			else
 				strcpy(jobname, " ");
 			continue;
 
 		case 'C':
 			if (line[1] != '\0')
-				strncpy(class, line+1, sizeof(class) - 1);
+				strncpy(class, line+1, sizeof(class)-1);
 			else if (class[0] == '\0')
 				gethostname(class, sizeof(class));
-			class[sizeof(class) - 1] = '\0';
 			continue;
 
 		case 'T':	/* header title for pr */
-			strncpy(title, line+1, sizeof(title) - 1);
-			title[sizeof(title) - 1] = '\0';
+			strncpy(title, line+1, sizeof(title)-1);
 			continue;
 
 		case 'L':	/* identification line */
@@ -445,21 +368,16 @@ printit(file)
 		case '2':
 		case '3':
 		case '4':
-			if (line[1] != '\0') {
-				strncpy(fonts[line[0]-'1'], line+1,
-					sizeof(fonts[0])-1);
-				fonts[line[0]-'1'][sizeof(fonts[0])-1] = '\0';
-			}
+			if (line[1] != '\0')
+				strcpy(fonts[line[0]-'1'], line+1);
 			continue;
 
 		case 'W':	/* page width */
-			strncpy(width+2, line+1, sizeof(width) - 3);
-			width[2+sizeof(width) - 3] = '\0';
+			strncpy(width+2, line+1, sizeof(width)-3);
 			continue;
 
 		case 'I':	/* indent amount */
-			strncpy(indent+2, line+1, sizeof(indent) - 3);
-			indent[2+sizeof(indent) - 3] = '\0';
+			strncpy(indent+2, line+1, sizeof(indent)-3);
 			continue;
 
 		default:	/* some file to print */
@@ -482,7 +400,6 @@ printit(file)
 		case 'N':
 		case 'U':
 		case 'M':
-		case 'R':
 			continue;
 		}
 
@@ -503,9 +420,7 @@ pass2:
 			continue;
 
 		case 'U':
-			if (strchr(line+1, '/'))
-				continue;
-			(void) unlinkfile(line+1, fromhost, logname);
+			(void) unlink(line+1);
 		}
 	/*
 	 * clean-up in case another control file exists
@@ -525,7 +440,6 @@ pass2:
  * Note: all filters take stdin as the file, stdout as the printer,
  * stderr as the log file, and must not ignore SIGINT.
  */
-static int
 print(format, file)
 	int format;
 	char *file;
@@ -538,33 +452,9 @@ print(format, file)
 	int pid, p[2], stopped = 0;
 	union wait status;
 	struct stat stb;
-	struct passwd *dude;
-	uid_t euid;
-	gid_t egid;
-	
-	euid = geteuid();
-	egid = getegid();
-	
-	if ((dude=getpwnam(euser)) == (struct passwd *)0 || !*euser)
-	        return(ERROR);
-	
-	if (setegid(dude->pw_gid) < 0)
-	  return (ERROR);
-	if (seteuid(dude->pw_uid) < 0)
-	  return (ERROR);
-	
-	/* Open the file as the user. */
 
-	if (lstat(file, &stb) < 0 || (fi = open(file, O_RDONLY)) < 0) {
-		seteuid(euid);
-		setegid(egid);
-		return(ACCESS);
-	}
-	
-	if (seteuid(euid) < 0 || setegid(egid) < 0) {
-		syslog(LOG_ERR, "can't reset effective uid/gid!");
-		return (ERROR);
-	}
+	if (lstat(file, &stb) < 0 || (fi = open(file, O_RDONLY)) < 0)
+		return(ERROR);
 	/*
 	 * Check to see if data file is a symbolic link. If so, it should
 	 * still point to the same file or someone is trying to print
@@ -577,7 +467,6 @@ print(format, file)
 		(void) write(ofd, FF, strlen(FF));
 		tof = 1;
 	}
-
 	if (IF == NULL && (format == 'f' || format == 'l')) {
 		tof = 0;
 		while ((n = read(fi, buf, BUFSIZ)) > 0)
@@ -602,7 +491,7 @@ print(format, file)
 			goto start;
 		}
 		pipe(p);
-		if ((prchild=doforkuser(DORETURN,euser)) == 0) { /* child */
+		if ((prchild = dofork(DORETURN)) == 0) {	/* child */
 			dup2(fi, 0);		/* file is stdin */
 			dup2(p[1], 1);		/* pipe is stdout */
 			for (n = 3; n < NOFILE; n++)
@@ -651,8 +540,7 @@ print(format, file)
 		} else {
 			for (n = 0; n < 4; n++) {
 				if (fonts[n][0] != '/')
-					(void) write(fo, _PATH_VFONT,
-					    sizeof(_PATH_VFONT) - 1);
+					(void) write(fo, _PATH_VFONT, 15);
 				(void) write(fo, fonts[n], strlen(fonts[n]));
 				(void) write(fo, "\n", 1);
 			}
@@ -687,14 +575,7 @@ print(format, file)
 			printer, format);
 		return(ERROR);
 	}
-	if (prog == NULL) {
-		(void) close(fi);
-		syslog(LOG_ERR,
-		   "%s: no filter found in printcap for format character '%c'",
-		   printer, format);
-		return(ERROR);
-	}
-	if ((av[0] = strrchr(prog, '/')) != NULL)
+	if ((av[0] = rindex(prog, '/')) != NULL)
 		av[0]++;
 	else
 		av[0] = prog;
@@ -702,19 +583,6 @@ print(format, file)
 	av[n++] = logname;
 	av[n++] = "-h";
 	av[n++] = fromhost;
-
-	/* Add Jobname */
-	if (jobname[0] && (jobname[0] != ' ')) {
-		av[n++] = "-j";
-		av[n++] = jobname;
-	}
-
-	/* Add Extended Parms */
-	if (extparms[0] && (extparms[0] != ' ')) {
-		av[n++] = "-z";
-        av[n++] = extparms;
-	}
-
 	av[n++] = AF;
 	av[n] = 0;
 	fo = pfd;
@@ -725,15 +593,14 @@ print(format, file)
 			;
 		if (!WIFSTOPPED(status)) {
 			(void) close(fi);
-			syslog(LOG_WARNING,
-				"%s: output filter died (retcode=%d termsig=%d)",
-				printer, status.w_retcode, status.w_termsig);
+			syslog(LOG_WARNING, "%s: output filter died (%d)",
+				printer, status.w_retcode);
 			return(REPRINT);
 		}
 		stopped++;
 	}
 start:
-	if ((child = doforkuser(DORETURN,euser)) == 0) {	/* child */
+	if ((child = dofork(DORETURN)) == 0) {	/* child */
 		dup2(fi, 0);
 		dup2(fo, 1);
 		n = open(tempfile, O_WRONLY|O_CREAT|O_TRUNC, 0664);
@@ -741,7 +608,6 @@ start:
 			dup2(n, 2);
 		for (n = 3; n < NOFILE; n++)
 			(void) close(n);
-		signal(SIGCHLD,SIG_DFL);
 		execv(prog, av);
 		syslog(LOG_ERR, "cannot execv %s", prog);
 		exit(2);
@@ -763,14 +629,16 @@ start:
 	tof = 0;
 
 	/* Copy filter output to "lf" logfile */
-	if ((fp = fopen(tempfile, "r"))) {
+	if (fp = fopen(tempfile, "r")) {
+		char tbuf[512];
+
 		while (fgets(buf, sizeof(buf), fp))
 			fputs(buf, stderr);
-		fclose(fp);
+		close(fp);
 	}
 
 	if (!WIFEXITED(status)) {
-		syslog(LOG_WARNING, "%s: filter '%c' terminated (termsig=%d)",
+		syslog(LOG_WARNING, "%s: Daemon filter '%c' terminated (%d)",
 			printer, format, status.w_termsig);
 		return(ERROR);
 	}
@@ -780,12 +648,11 @@ start:
 		return(OK);
 	case 1:
 		return(REPRINT);
+	default:
+		syslog(LOG_WARNING, "%s: Daemon filter '%c' exited (%d)",
+			printer, format, status.w_retcode);
 	case 2:
 		return(ERROR);
-	default:
-		syslog(LOG_WARNING, "%s: filter '%c' exited (retcode=%d)",
-			printer, format, status.w_retcode);
-		return(FILTERERR);
 	}
 }
 
@@ -794,52 +661,17 @@ start:
  * Return -1 if a non-recoverable error occured, 1 if a recoverable error and
  * 0 if all is well.
  */
-static int
 sendit(file)
 	char *file;
 {
 	register int i, err = OK;
 	char *cp, last[BUFSIZ];
-	char *dup_cfpname = NULL;
-	char *final_cfp;
-	FILE *dup_cfp = NULL;
-	struct stat stb;
 
-	int  ifi, fi;
-	int  ifchild, pid;
-	int  sendresult;
-	int  tmpmask;
-
-	int  ii;
-    
-    
 	/*
 	 * open control file
 	 */
 	if ((cfp = fopen(file, "r")) == NULL)
 		return(OK);
-
-	/* Don't duplicate the cfpname, becuase if it doesn't
-	 * get deleted, weird stuff will happen. */
-	dup_cfpname=strdup("lpdtempXXXXXX");
-	if ((ii=mkstemp(dup_cfpname))==-1) {
-		syslog(LOG_ERR, "Can't create temp cfp file");
-		return(OK);
-	}
-	close(ii);
-	tmpmask = umask(007);
-	if (rename(file,dup_cfpname)==-1) {
-		syslog(LOG_ERR, "Can't rename cfp file");
-		return(OK);
-	}
-	dup_cfp=fopen(file, "w");
-	umask(tmpmask);
-	if (dup_cfp == NULL)
-	{
-		syslog(LOG_ERR, "Can't create temp cfp file %s", dup_cfpname);
-		return(OK);
-	}
-	    
 	/*
 	 *      read the control file for work to do
 	 *
@@ -848,7 +680,6 @@ sendit(file)
 	 *      commands of interest are:
 	 *
 	 *            a-z -- "file name" name of file to print
-	 *		P -- "person" user's login name
 	 *              U -- "unlink" name of file to remove
 	 *                    (after we print it. (Pass 2 only)).
 	 */
@@ -858,9 +689,7 @@ sendit(file)
 	 */
 	while (getline(cfp)) {
 	again:
-	         if (line[0] == 'S') {
-		        if (dup_cfp)
-			  fprintf(dup_cfp,"%s\n",line);
+		if (line[0] == 'S') {
 			cp = line+1;
 			i = 0;
 			while (*cp >= '0' && *cp <= '9')
@@ -873,154 +702,12 @@ sendit(file)
 			fino = i;
 			continue;
 		}
-		if (line[0] == 'P') {
-		        if (dup_cfp)
-			  fprintf(dup_cfp,"%s\n",line);
-
-			strncpy(logname, line+1, sizeof(logname)-1);
-			logname[sizeof(logname) - 1] = '\0';
-			if (RS) {                       /* restricted */
-				if (getpwnam(logname) == (struct passwd *)0) {
-					sendmail(line+1, NOACCT);
-					goto pass2;
-				}
-			} else {			/* not restricted */
-				struct passwd *dude;
-				dude = getpwnam(logname);
-				if (dude == NULL || dude->pw_uid == 0) {
-				   /* if user dne, set euid to lp */
-				   strncpy(euser, "lp", 3);
-				} else {
-				   strncpy(euser, logname, sizeof(logname));
-				}
-			}
-			continue;
-		}
-
-		/* a hack */
-		if (line[0] < 'a' || line[0] > 'z') {
-			
-		  if (dup_cfp && line[0] != 'U') {
-			  if (!SH) {
-				  fprintf(dup_cfp,"%s\n",line);
-			  } else if (line[0] != 'J' && line[0] != 'C' && line[0] != 'L') {
-				  fprintf(dup_cfp,"%s\n",line);
-			  }
-		  }
-		  continue;
-		}
-		  
-		/* we assume that we've hit a line dedicated to printing */
 		if (line[0] >= 'a' && line[0] <= 'z') {
-		        char tmpfile[20];
-			
-			memset(tmpfile,'\0',20);
-			if (dup_cfp)
-			  fprintf(dup_cfp, "%s\n", line);
-
-		        strcpy(last, line);
-			/* handle multiple copies */
-                       while (i = getline(cfp)) {
-                         if (dup_cfp)
-                           fprintf(dup_cfp, "%s\n", line);
+			strcpy(last, line);
+			while (i = getline(cfp))
 				if (strcmp(last, line))
 					break;
-                       }
-			
-
-			/* msf - here is where we hack in IF support    */
-			/*       will only run the IF, no other filters */
-			/*       will be run.                           */
-                       /* This hack will work for multiple files only  */
-                       /* if the 'f' lines are separated by at least   */
-                       /* one non-'f' line (i.e. a U or N line) due to */
-                       /* the multiple copies getline() while loop     */
-                       /* right above this comment.                    */
-			if (IF != NULL && last[0] == 'f') {
-			    struct passwd *dude;
-			    uid_t euid;
-			    gid_t egid;
-			  
-			    memcpy(tmpfile, last+1, 7);
-  			    memcpy(tmpfile+7, "XXXXXX", 6);
-			    tmpfile[13]='\000';
-			    mktemp(tmpfile);
-
-			    /* Nasty trick of the day, to attempt
-			     * to be RFC compliant */
-			    if (rename(last+1,tmpfile)==-1) {
-				    syslog(LOG_ERR,"can't rename data file");
-				    return(ERROR);
-			    }
-				
-			    euid = geteuid();
-			    egid = getegid();
-	
-			    /* Make sure we open the file as the user */
-			    if ((dude=getpwnam(euser)) == (struct passwd *)0 || !*euser) {
-				    syslog(LOG_ERR,"don't know who to print as!");
-				    return(ERROR);
-			    }
-	
-			    if (setegid(dude->pw_gid) < 0)
-				    return(ERROR);
-			    if (seteuid(dude->pw_uid) < 0)
-				    return(ERROR);
-
-			     fi = open(tmpfile, O_RDONLY);
-				
-			     /* Ideally here, if the open fails, we'd
-			      * just bail. However, doing this instead
-			      * allows us to clean up the queue nicely. 
-			      */
-			     if (fi==-1 ||
-				 ((stb.st_mode & S_IFMT) == S_IFLNK 
-				  && fstat(fi, &stb) == 0 &&
-				  (stb.st_dev != fdev || stb.st_ino != fino))
-				 ) {
-				     syslog(LOG_ERR, "%s: couldn't open spool file %s as %s",
-					    name, last+1, euser);
-				     fi = open("/dev/null",O_RDONLY);
-			     }
-			     if (seteuid(euid) < 0 || setegid(egid) < 0) {
-				     syslog(LOG_ERR, "%s: couldn't reset effective uid/gid!\n", name);
-				     return(ERROR);
-			     }
-				
-			    ifi = open(last+1,O_WRONLY|O_CREAT|O_TRUNC, 0664);
-			    if ((ifchild = doforkuser(DORETURN,euser)) == 0) { /*child*/
-			         dup2(fi,  0); /* input file is stdin  */
-  			         dup2(ifi, 1); /* temp file is  stdout */
-			         for (ii = 3; ii < NOFILE; ii++)
-			            (void) close(ii);
-
-			         ii=execl(IF, IF, 0);
-			         syslog(LOG_ERR, "cannot execv %s", IF);
-			         exit(2);
-			    }
-			  close(ifi);
-                          close(fi);
-			  /* wait on it to finish */
-			  signal(SIGCHLD,SIG_DFL);
-			  waitpid(ifchild, NULL, 0);
-			 }
-		      
-                        sendresult = sendfile('\3', last+1);
-
-    		        if ( IF != NULL && last[0] == 'f')
-			  {
-			     if (unlink(last+1) != 0) {
-				     syslog(LOG_ERR, "error unlinking %s: %d",
-					    last+1,errno);
-			     }
-			     if (rename(tmpfile,last+1)==-1) {
-				     syslog(LOG_ERR, "error renaming %s: %d",
-					    tmpfile, errno);
-			     }
-			  }
-
-			/* clean up our temp file */
-			switch (sendresult) {
+			switch (sendfile('\3', last+1)) {
 			case OK:
 				if (i)
 					goto again;
@@ -1036,61 +723,22 @@ sendit(file)
 			break;
 		}
 	}
-
-	if (dup_cfp)
-	  {
-	    fclose(dup_cfp);
-	    dup_cfp = NULL;
-	  }
-        final_cfp=file;
-
-	if (err == OK && sendfile('\2', final_cfp) > 0) {
+	if (err == OK && sendfile('\2', file) > 0) {
 		(void) fclose(cfp);
-		if (dup_cfp)
-		  {
-		    (void) fclose(dup_cfp);
-		    unlink(file);
-		    rename(dup_cfpname,file);
-		    free(dup_cfpname);
-		  }
 		return(REPRINT);
 	}
-pass2:
 	/*
 	 * pass 2
 	 */
-	if (dup_cfp)
-	  {
-	    final_cfp=dup_cfpname;
-	    fclose(dup_cfp);
-	    dup_cfp = NULL;
-	  }
-	else
-	    final_cfp=file;
-
- 	fclose(cfp);
-
-	if ((cfp = fopen(final_cfp, "r")) == NULL) {
-		if (dup_cfpname) {
-		    unlink(dup_cfpname);
-		    free(dup_cfpname);
-		}
-		return(OK);
-	}
-
+	fseek(cfp, 0L, 0);
 	while (getline(cfp))
-		if (line[0] == 'U' && strchr(line+1, '/') == NULL)
-			(void) unlinkfile(line+1, from, logname);
+		if (line[0] == 'U')
+			(void) unlink(line+1);
 	/*
 	 * clean-up in case another control file exists
 	 */
 	(void) fclose(cfp);
-	unlink(file);
-	if (dup_cfpname) {
-	    unlink(dup_cfpname);
-	    free(dup_cfpname);
-	}
-
+	(void) unlink(file);
 	return(err);
 }
 
@@ -1098,45 +746,16 @@ pass2:
  * Send a data file to the remote machine and spool it.
  * Return positive if we should try resending.
  */
-static int
 sendfile(type, file)
-	int type;
-	char *file;
+	char type, *file;
 {
 	register int f, i, amt;
 	struct stat stb;
 	char buf[BUFSIZ];
 	int sizerr, resp;
-	struct passwd *dude;
-	uid_t euid;
-	gid_t egid;
-	
-	euid = geteuid();
-	egid = getegid();
-	
-	if (strncmp("cf",file,2)) {
-		if ((dude=getpwnam(euser)) == (struct passwd *)0 || !*euser)
-		  return(ERROR);
-	
-		if (setegid(dude->pw_gid) < 0)
-		  return (ERROR);
-		if (seteuid(dude->pw_uid) < 0)
-		  return (ERROR);
-	}
-	
-	/* Open the file as the user. */
-
-	if (lstat(file, &stb) < 0 || (f = open(file, O_RDONLY)) < 0) {
-		seteuid(euid);
-		setegid(egid);
-		return(ACCESS);
-	}
-	
-	if (seteuid(euid) < 0 || setegid(egid) < 0) {
-		syslog(LOG_ERR, "can't reset effective uid/gid!");
-		return (ERROR);
-	}
 
+	if (lstat(file, &stb) < 0 || (f = open(file, O_RDONLY)) < 0)
+		return(ERROR);
 	/*
 	 * Check to see if data file is a symbolic link. If so, it should
 	 * still point to the same file or someone is trying to print something
@@ -1145,7 +764,7 @@ sendfile(type, file)
 	if ((stb.st_mode & S_IFMT) == S_IFLNK && fstat(f, &stb) == 0 &&
 	    (stb.st_dev != fdev || stb.st_ino != fino))
 		return(ACCESS);
-	(void) snprintf(buf, sizeof(buf), "%c%d %s\n", type, stb.st_size, file);
+	(void) sprintf(buf, "%c%d %s\n", type, stb.st_size, file);
 	amt = strlen(buf);
 	for (i = 0;  ; i++) {
 		if (write(pfd, buf, amt) != amt ||
@@ -1155,14 +774,14 @@ sendfile(type, file)
 		} else if (resp == '\0')
 			break;
 		if (i == 0)
-			pstatus("no space on remote; waiting for queue to drain");
+			status("no space on remote; waiting for queue to drain");
 		if (i == 10)
 			syslog(LOG_ALERT, "%s: can't send to %s; queue full",
 				printer, RM);
 		sleep(5 * 60);
 	}
 	if (i)
-		pstatus("sending to %s", RM);
+		status("sending to %s", RM);
 	sizerr = 0;
 	for (i = 0; i < stb.st_size; i += BUFSIZ) {
 		amt = BUFSIZ;
@@ -1192,7 +811,6 @@ sendfile(type, file)
  * are in sync with eachother.
  * Return non-zero if the connection was lost.
  */
-static char
 response()
 {
 	char resp;
@@ -1207,11 +825,11 @@ response()
 /*
  * Banner printing stuff
  */
-static void
 banner(name1, name2)
 	char *name1, *name2;
 {
 	time_t tvec;
+	extern char *ctime();
 
 	time(&tvec);
 	if (!SF && !tof)
@@ -1247,11 +865,10 @@ banner(name1, name2)
 	tof = 1;
 }
 
-static char *
+char *
 scnline(key, p, c)
-	register int key;
-	register char *p;
-	int c;
+	register char key, *p;
+	char c;
 {
 	register scnwidth;
 
@@ -1264,10 +881,9 @@ scnline(key, p, c)
 
 #define TRC(q)	(((q)-' ')&0177)
 
-static void
 scan_out(scfd, scsp, dlm)
-	int scfd, dlm;
-	char *scsp;
+	int scfd;
+	char *scsp, dlm;
 {
 	register char *strp;
 	register nchrs, j;
@@ -1293,14 +909,13 @@ scan_out(scfd, scsp, dlm)
 		while (*--strp == BACKGND && strp >= outbuf)
 			;
 		strp++;
-		*strp++ = '\n';
+		*strp++ = '\n';	
 		(void) write(scfd, outbuf, strp-outbuf);
 	}
 }
 
-static int
 dropit(c)
-	int c;
+	char c;
 {
 	switch(c) {
 
@@ -1319,39 +934,10 @@ dropit(c)
 	}
 }
 
-/*
- * Safely (I hope) unlink a file.
- */
-static void
-unlinkfile(file, host, user)
-	char *file;
-	char *host;
-	char *user;
-{
-	/* this hack duplicates some code but things should work better */
-	/* if the file does not exist, do not worry about doing unlink */
-	int ret;
-	extern int errno;
-	if ((strchr(file,'/') == NULL) &&
-	    (strstr(file,"..") == NULL)) {
-		ret = unlink(file);
-		if (ret == 0)
-			return;
-		if (errno == 2)
-			return;
-		syslog(LOG_ERR,"unlink file=%s, ret=%i, errno=%i",
-			file, ret, errno);
-	}
-	
-	if (dounlink(file, host, user))
-		sendmail(user, UNLINK);
-}
-
 /*
  * sendmail ---
  *   tell people about job completion
  */
-static void
 sendmail(user, bombed)
 	char *user;
 	int bombed;
@@ -1359,6 +945,7 @@ sendmail(user, bombed)
 	register int i;
 	int p[2], s;
 	register char *cp;
+	char buf[100];
 	struct stat stb;
 	FILE *fp;
 
@@ -1367,54 +954,44 @@ sendmail(user, bombed)
 		dup2(p[0], 0);
 		for (i = 3; i < NOFILE; i++)
 			(void) close(i);
-		if ((cp = strrchr(_PATH_SENDMAIL, '/')) != NULL)
+		if ((cp = rindex(_PATH_SENDMAIL, '/')) != NULL)
 			cp++;
 		else
 			cp = _PATH_SENDMAIL;
-		execl(_PATH_SENDMAIL, cp, "-t", 0);
+		sprintf(buf, "%s@%s", user, fromhost);
+		execl(_PATH_SENDMAIL, cp, buf, 0);
 		exit(0);
 	} else if (s > 0) {				/* parent */
 		dup2(p[1], 1);
 		printf("To: %s@%s\n", user, fromhost);
-		printf("Subject: %s printer job \"%s\"\n", printer,
-			*jobname ? jobname : "<unknown>");
-		printf("Reply-To: root@%s\n\n", host);
+		printf("Subject: printer job\n\n");
 		printf("Your printer job ");
 		if (*jobname)
 			printf("(%s) ", jobname);
 		switch (bombed) {
 		case OK:
 			printf("\ncompleted successfully\n");
-			cp = "OK";
 			break;
 		default:
 		case FATALERR:
 			printf("\ncould not be printed\n");
-			cp = "FATALERR";
 			break;
 		case NOACCT:
 			printf("\ncould not be printed without an account on %s\n", host);
-			cp = "NOACCT";
 			break;
 		case FILTERERR:
 			if (stat(tempfile, &stb) < 0 || stb.st_size == 0 ||
 			    (fp = fopen(tempfile, "r")) == NULL) {
-				printf("\nhad some errors and may not have printed\n");
+				printf("\nwas printed but had some errors\n");
 				break;
 			}
 			printf("\nwas printed but had the following errors:\n");
 			while ((i = getc(fp)) != EOF)
 				putchar(i);
 			(void) fclose(fp);
-			cp = "FILTERERR";
 			break;
 		case ACCESS:
 			printf("\nwas not printed because it was not linked to the original file\n");
-			cp = "ACCESS";
-			break;
-		case UNLINK:
-			printf("\nwas not unlinked: permission denied\n");
-			break;
 		}
 		fflush(stdout);
 		(void) close(1);
@@ -1427,7 +1004,6 @@ sendmail(user, bombed)
 /*
  * dofork - fork with retries on failure
  */
-static int
 dofork(action)
 	int action;
 {
@@ -1459,92 +1035,11 @@ dofork(action)
 	/*NOTREACHED*/
 }
 
-
-/*
- * doforkuser - fork with retries on failure
- *          sets up as user first for security 
- */
-static int
-doforkuser(action, user)
-	int action;
-	char *user;
-{
-	register int i, pid;
-	struct passwd *dude;
-	struct group *lpgrp;
-	
-	/* figure out who we wish we were */
-	dude = getpwnam(user);
-	if (!dude) {
-	  syslog(LOG_ERR, "can't find user - can't fork");
-
-	  switch (action) {
-	    case DORETURN:
-	      return (-1);
-	    default:
-	      syslog(LOG_ERR, "bad action (%d) to dofork", action);
-	      /*FALL THRU*/
-	    case DOABORT:
-	      exit(1);
-	  }
-	}
-	
-	/* This shouldn't fail */
-	lpgrp = getgrnam("lp");
-	if (!lpgrp) {
-		syslog(LOG_ERR, "can't find lp group - can't fork");
-		switch (action) {
-		 case DORETURN:
-			return (-1);
-		 default:
-			syslog(LOG_ERR, "bad action (%d) to dofork", action);
-		 case DOABORT:
-			exit(1);
-		}
-	}
-	
-	for (i = 0; i < 20; i++) {
-		if ((pid = fork()) < 0) {
-			sleep((unsigned)(i*i));
-			continue;
-		}
-		/*
-		 * Child should run as user instead of root
-		 * Add to lp group so it can read config file.
-		 */
-		if (pid == 0) {
-			unsetenv("BASH_ENV");
-			unsetenv("ENV");
-			setenv("USER",user,1);
-			setgroups(1,&lpgrp->gr_gid);
-			setgid(dude->pw_gid);
-			setuid(dude->pw_uid);
-		}
-		return(pid);
-	}
-	syslog(LOG_ERR, "can't fork");
-
-	switch (action) {
-	case DORETURN:
-		return (-1);
-	default:
-		syslog(LOG_ERR, "bad action (%d) to dofork", action);
-		/*FALL THRU*/
-	case DOABORT:
-		exit(1);
-	}
-	/*NOTREACHED*/
-}
-
-
-
-
 /*
  * Kill child processes to abort current job.
  */
-static void
-abortpr(signo)
-	int signo;
+void
+abortpr()
 {
 	(void) unlink(tempfile);
 	kill(0, SIGINT);
@@ -1555,7 +1050,6 @@ abortpr(signo)
 	exit(0);
 }
 
-static void
 init()
 {
 	int status;
@@ -1618,67 +1112,22 @@ init()
 	HL = pgetflag("hl");
 	RW = pgetflag("rw");
 	BR = pgetnum("br");
-
-/* FIX for hardware handshaking.  DJM
-   Added new function for these unsigned longs
-*/
-	FC = pgetusgn("fc");
-	FS = pgetusgn("fs");
-	XC = pgetusgn("xc");
-	XS = pgetusgn("xs");
-
+	if ((FC = pgetnum("fc")) < 0)
+		FC = 0;
+	if ((FS = pgetnum("fs")) < 0)
+		FS = 0;
 	tof = !pgetflag("fo");
 }
 
 /*
  * Acquire line printer or remote connection.
  */
-static void
 openpr()
 {
 	register int i, n;
 	int resp;
-	char *cp;
 
 	if (!sendtorem && *LP) {
-               if ((cp = strchr(LP, '@')) != NULL) {
-                       char savech;
-                       struct servent *sp;
-                       struct hostent *hp;
-                       struct sockaddr_in sin;
-
-                       savech = *cp;
-                       *cp = '\0';
-                       if ((sp = getservbyname(LP, "tcp")) != NULL) {
-                               sin.sin_port = sp->s_port;
-                       } else if (isdigit(*LP)) {
-                               sin.sin_port = htons(atoi(LP));
-                       } else {
-                               syslog(LOG_ERR, "%s: bad service spec", LP);
-                               exit(1);
-                       }
-                       *cp = savech;
-                       if (inet_aton(cp+1, &sin.sin_addr))
-                               NULL;
-                       else if ((hp = gethostbyname(cp+1)) != NULL)
-                               sin.sin_addr.s_addr = *(u_long *)hp->h_addr;
-                       else {
-                               syslog(LOG_ERR, "%s: bad host spec", cp+1);
-                               exit(1);
-                       }
-                       if ((pfd = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) <0){
-                               syslog(LOG_ERR, "socket: %m");
-                               exit(1);
-                       }
-#ifndef __linux__
-                       sin.sin_len = sizeof sin;
-#endif
-                       sin.sin_family = AF_INET;
-                       if (connect(pfd, (struct sockaddr*)&sin, sizeof sin) <0){
-                               syslog(LOG_ERR, "connect: %m");
-                               exit(1);
-                       }
-               } else {
 		for (i = 1; ; i = i < 32 ? i << 1 : i) {
 			pfd = open(LP, RW ? O_RDWR : O_WRONLY);
 			if (pfd >= 0)
@@ -1688,44 +1137,35 @@ openpr()
 				exit(1);
 			}
 			if (i == 1)
-				pstatus("waiting for %s to become ready (offline ?)", printer);
+				status("waiting for %s to become ready (offline ?)", printer);
 			sleep(i);
 		}
 		if (isatty(pfd))
 			setty();
-		}
-		pstatus("%s is ready and printing", printer);
+		status("%s is ready and printing", printer);
 	} else if (RM != NULL) {
 		for (i = 1; ; i = i < 256 ? i << 1 : i) {
 			resp = -1;
 			pfd = getport(RM);
 			if (pfd >= 0) {
-				char c;
-				
-				(void) snprintf(line, sizeof(line), "\2%s\n", RP);
+				(void) sprintf(line, "\2%s\n", RP);
 				n = strlen(line);
 				if (write(pfd, line, n) == n &&
 				    (resp = response()) == '\0')
 					break;
-#ifdef SHUT_WR
-				shutdown(pfd, SHUT_WR);
-#else
-				shutdown(pfd, 1);
-#endif
-				read(pfd , &c, 1);
 				(void) close(pfd);
 			}
 			if (i == 1) {
 				if (resp < 0)
-					pstatus("waiting for %s to come up", RM);
+					status("waiting for %s to come up", RM);
 				else {
-					pstatus("waiting for queue to be enabled on %s", RM);
+					status("waiting for queue to be enabled on %s", RM);
 					i = 256;
 				}
 			}
 			sleep(i);
 		}
-		pstatus("sending to %s", RM);
+		status("sending to %s", RM);
 		remote = 1;
 	} else {
 		syslog(LOG_ERR, "%s: no line printer device or host name",
@@ -1737,6 +1177,7 @@ openpr()
 	 */
 	if (!remote && OF) {
 		int p[2];
+		char *cp;
 
 		pipe(p);
 		if ((ofilter = dofork(DOABORT)) == 0) {	/* child */
@@ -1744,7 +1185,7 @@ openpr()
 			dup2(pfd, 1);		/* printer is std out */
 			for (i = 3; i < NOFILE; i++)
 				(void) close(i);
-			if ((cp = strrchr(OF, '/')) == NULL)
+			if ((cp = rindex(OF, '/')) == NULL)
 				cp = OF;
 			else
 				cp++;
@@ -1785,7 +1226,6 @@ struct bauds {
 /*
  * setup tty lines.
  */
-static void
 setty()
 {
 	struct termios ttybuf;
@@ -1813,29 +1253,12 @@ setty()
 	}
 }
 
-#ifdef	__STDC__
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
-
-static void
-#ifdef	__STDC__
-pstatus(const char *msg, ...)
-#else
-pstatus(msg, va_alist)
+/*VARARGS1*/
+status(msg, a1, a2, a3)
 	char *msg;
-        va_dcl
-#endif
 {
 	register int fd;
 	char buf[BUFSIZ];
-	va_list ap;
-#ifdef __STDC__
-	va_start(ap, msg);
-#else
-	va_start(ap);
-#endif
 
 	umask(0);
 	fd = open(ST, O_WRONLY|O_CREAT, 0664);
@@ -1844,16 +1267,8 @@ pstatus(msg, va_alist)
 		exit(1);
 	}
 	ftruncate(fd, 0);
-	(void)vsnprintf(buf, sizeof(buf) - 1, msg, ap);
-	va_end(ap);
+	sprintf(buf, msg, a1, a2, a3);
 	strcat(buf, "\n");
 	(void) write(fd, buf, strlen(buf));
 	(void) close(fd);
 }
-
-void
-alarmhandler(signo)
-	int signo;
-{
-	/* ignored */
-}
