@@ -37,6 +37,10 @@ char copyright[] =
  All rights reserved.\n";
 #endif /* not lint */
 
+#ifndef lint
+/*static char sccsid[] = "from: @(#)lpr.c	5.9 (Berkeley) 3/2/91";*/
+static char rcsid[] = "$Id: lpr.c,v 1.4 1993/08/01 17:58:40 mycroft Exp $";
+#endif /* not lint */
 /*
  *      lpr -- off line print
  *
@@ -44,7 +48,6 @@ char copyright[] =
  * using information from a printer data base.
  */
 
-#include <errno.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/file.h>
@@ -57,11 +60,10 @@ char copyright[] =
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <netdb.h>
 #include "lp.local.h"
 #include "pathnames.h"
 
-#define MAXHOSTNAMELEN 128
-
 char    *tfname;		/* tmp copy of cf before linking */
 char    *cfname;		/* daemon control files, linked from tf's */
 char    *dfname;		/* data files */
@@ -83,7 +85,7 @@ char	*person;		/* user name */
 char	*title;			/* pr'ing title */
 char	*fonts[4];		/* troff font names */
 char	*width;			/* width for versatec printing */
-char	host[MAXHOSTNAMELEN];		/* host name */
+char	host[32];		/* host name */
 char	*class = host;		/* class title on header page */
 char    *jobname;		/* job name on header page */
 char	*name;			/* program name */
@@ -98,9 +100,6 @@ char	*LO;			/* lock file name */
 char	*RG;			/* restrict group */
 static short	SC;			/* suppress multiple copies */
 
-uid_t	euid,uid;
-gid_t	egid,gid;
-
 char	*getenv();
 char	*rindex();
 char	*linked();
@@ -118,13 +117,6 @@ main(argc, argv)
 	char buf[BUFSIZ];
 	int i, f;
 	struct stat stb;
-	
-	euid = geteuid();
-	egid = getegid();
-	gid = getgid();
-	uid = getuid();
-	setegid(gid);
-	seteuid(uid);
 
 	if (signal(SIGHUP, SIG_IGN) != SIG_IGN)
 		signal(SIGHUP, cleanup);
@@ -137,7 +129,14 @@ main(argc, argv)
 
 	name = argv[0];
 	gethostname(host, sizeof (host));
-	host[MAXHOSTNAMELEN-1]='\0';
+	{ /* use the official host name, like lprm */
+		register struct hostent *hp;
+
+		if (hp = gethostbyname(host))
+			strcpy(host, hp->h_name);
+		else
+			fatal("unable to get official name for local machine");
+	}
 	openlog("lpd", 0, LOG_LPR);
 
 	while (argc > 1 && argv[1][0] == '-') {
@@ -266,7 +265,7 @@ main(argc, argv)
 	 * Get the identity of the person doing the lpr using the same
 	 * algorithm as lprm. 
 	 */
-	userid = uid;
+	userid = getuid();
 	if (userid != DU || person == 0) {
 		if ((pw = getpwuid(userid)) == NULL)
 			fatal("Who are you?");
@@ -278,7 +277,7 @@ main(argc, argv)
 	if (RG != NULL && userid != DU) {
 		if ((gptr = getgrnam(RG)) == NULL)
 			fatal("Restricted group specified incorrectly");
-		if (gptr->gr_gid != gid) {
+		if (gptr->gr_gid != getgid()) {
 			while (*gptr->gr_mem != NULL) {
 				if ((strcmp(person, *gptr->gr_mem)) == 0)
 					break;
@@ -291,7 +290,7 @@ main(argc, argv)
 	/*
 	 * Check to make sure queuing is enabled if userid is not root.
 	 */
-	(void) snprintf(buf, sizeof(buf), "%s/%s", SD, LO);
+	(void) sprintf(buf, "%s/%s", SD, LO);
 	if (userid && stat(buf, &stb) == 0 && (stb.st_mode & 010))
 		fatal("Printer queue is disabled");
 	/*
@@ -299,11 +298,7 @@ main(argc, argv)
 	 */
 	mktemps();
 	tfd = nfile(tfname);
-	seteuid(euid);
-	setegid(egid);
 	(void) fchown(tfd, DU, -1);	/* owned by daemon for protection */
-	setegid(gid);
-	seteuid(uid);
 	card('H', host);
 	card('P', person);
 	if (hdr) {
@@ -327,7 +322,7 @@ main(argc, argv)
 				card('1'+i, fonts[i]);
 	if (width != NULL)
 		card('W', width);
-	
+
 	/*
 	 * Read the files and spool them.
 	 */
@@ -338,8 +333,7 @@ main(argc, argv)
 			continue;	/* file unreasonable */
 
 		if (sflag && (cp = linked(arg)) != NULL) {
-			(void) snprintf(buf, sizeof(buf), "%ld %ld",
-				   (long) statb.st_dev, (long) statb.st_ino);
+			(void) sprintf(buf, "%d %d", statb.st_dev, statb.st_ino);
 			card('S', buf);
 			if (format == 'p')
 				card('T', title ? title : arg);
@@ -355,15 +349,14 @@ main(argc, argv)
 		}
 		if (sflag)
 			printf("%s: %s: not linked, copying instead\n", name, arg);
-		
 		if ((i = open(arg, O_RDONLY)) < 0) {
 			printf("%s: cannot open %s\n", name, arg);
-		} else {
-			copy(i, arg);
-			(void) close(i);
-			if (f && unlink(arg) < 0)
-			  printf("%s: %s: not removed\n", name, arg);
+			continue;
 		}
+		copy(i, arg);
+		(void) close(i);
+		if (f && unlink(arg) < 0)
+			printf("%s: %s: not removed\n", name, arg);
 	}
 
 	if (nact) {
@@ -372,8 +365,6 @@ main(argc, argv)
 		/*
 		 * Touch the control file to fix position in the queue.
 		 */
-		seteuid(euid);
-		setegid(egid);
 		if ((tfd = open(tfname, O_RDWR)) >= 0) {
 			char c;
 
@@ -395,8 +386,6 @@ main(argc, argv)
 			exit(0);
 		if (!startdaemon(printer))
 			printf("jobs queued, but cannot start daemon.\n");
-		setegid(gid);
-		seteuid(uid);
 		exit(0);
 	}
 	cleanup();
@@ -453,10 +442,9 @@ linked(file)
 {
 	register char *cp;
 	static char buf[BUFSIZ];
-	int ret;
 
 	if (*file != '/') {
-		if (getcwd(buf, BUFSIZ) == NULL)
+		if (getwd(buf) == NULL)
 			return(NULL);
 		while (file[0] == '.') {
 			switch (file[1]) {
@@ -473,16 +461,11 @@ linked(file)
 			}
 			break;
 		}
-		strncat(buf, "/", sizeof(buf) - strlen(buf) - 1);
-		strncat(buf, file, sizeof(buf) - strlen(buf) - 1);
+		strcat(buf, "/");
+		strcat(buf, file);
 		file = buf;
 	}
-	seteuid(euid);
-	setegid(egid);
-	ret=symlink(file, dfname);
-	setegid(gid);
-	seteuid(uid);
-	return(ret ? NULL : file);
+	return(symlink(file, dfname) ? NULL : file);
 }
 
 /*
@@ -495,14 +478,14 @@ card(c, p2)
 	register char *p1 = buf;
 	register int len = 2;
 
-        if (strlen(p2) > BUFSIZ-2)                     /* !! */
-        {                                              /* !! */
-                printf("Buffer overflow\n");	       /* !! */
-                exit(1);                               /* !! */
-        }
-
+	if (strlen(p2) > BUFSIZ-2)                     
+	{
+		printf("No, thanks...\n");
+		exit(1);
+	}
+		
 	*p1++ = c;
-	while ((c = *p2++) != '\0' && len < sizeof(buf)) {
+	while ((c = *p2++) != '\0') {
 		*p1++ = (c == '\n') ? ' ' : c;
 		len++;
 	}
@@ -518,9 +501,7 @@ nfile(n)
 {
 	register f;
 	int oldumask = umask(0);		/* should block signals */
-	
-	seteuid(euid);
-	setegid(egid);
+
 	f = creat(n, FILMOD);
 	(void) umask(oldumask);
 	if (f < 0) {
@@ -531,8 +512,6 @@ nfile(n)
 		printf("%s: cannot chown %s\n", name, n);
 		cleanup();
 	}
-	setegid(gid);
-	seteuid(uid);
 	if (++n[inchar] > 'z') {
 		if (++n[inchar-2] == 't') {
 			printf("too many files - break up the job\n");
@@ -557,8 +536,6 @@ cleanup()
 	signal(SIGQUIT, SIG_IGN);
 	signal(SIGTERM, SIG_IGN);
 	i = inchar;
-	seteuid(euid);
-	setegid(egid);
 	if (tfname)
 		do
 			unlink(tfname);
@@ -585,9 +562,10 @@ cleanup()
 test(file)
 	char *file;
 {
+	struct exec execb;
 	register int fd;
 	register char *cp;
-	
+
 	if (access(file, 4) < 0) {
 		printf("%s: cannot access %s\n", name, file);
 		return(-1);
@@ -608,8 +586,37 @@ test(file)
 		printf("%s: cannot open %s\n", name, file);
 		return(-1);
 	}
+#ifndef	__linux__
+	if (read(fd, &execb, sizeof(execb)) == sizeof(execb)) {
+		if(!N_BADMAG(execb) || 
+#ifdef SARMAG
+		    strncmp((char *)&execb, ARMAG, SARMAG)==0
+#else
+		    execb.a_magic==ARMAG
+#endif
+		    ) {
+			printf("%s: %s is an executable program", name, file);
+			goto error1;
+		}
+#ifdef A_MAGIC1
+		switch(execb.a_magic) {
+		case A_MAGIC1:
+		case A_MAGIC2:
+		case A_MAGIC3:
+#ifdef A_MAGIC4
+		case A_MAGIC4:
+#endif
+			printf("%s: %s is an executable program", name, file);
+			goto error1;
+
+		case ARMAG:
+			printf("%s: %s is an archive file", name, file);
+			goto error1;
+		}
+#endif
+	}
+#endif
 	(void) close(fd);
-#if 0
 	if (rflag) {
 		if ((cp = rindex(file, '/')) == NULL) {
 			if (access(".", 2) == 0)
@@ -624,9 +631,6 @@ test(file)
 		printf("%s: %s: is not removable by you\n", name, file);
 	}
 	return(0);
-#else
-	return rflag;
-#endif
 
 error1:
 	printf(" and is unprintable\n");
@@ -643,9 +647,6 @@ itoa(i)
 {
 	static char b[10] = "########";
 	register char *p;
-	
-	if ((i<0) || (i > 99999999))
-	  i=0;
 
 	p = &b[8];
 	do
@@ -693,9 +694,7 @@ mktemps()
 	register char *cp;
 	char buf[BUFSIZ];
 
-	(void) snprintf(buf, sizeof(buf), "%s/.seq", SD);
-	seteuid(euid);
-	setegid(egid);
+	(void) sprintf(buf, "%s/.seq", SD);
 	if ((fd = open(buf, O_RDWR|O_CREAT, 0661)) < 0) {
 		printf("%s: cannot create %s\n", name, buf);
 		exit(1);
@@ -704,8 +703,6 @@ mktemps()
 		printf("%s: cannot lock %s\n", name, buf);
 		exit(1);
 	}
-	setegid(gid);
-	seteuid(uid);
 	n = 0;
 	if ((len = read(fd, buf, sizeof(buf))) > 0) {
 		for (cp = buf; len--; ) {
@@ -714,10 +711,12 @@ mktemps()
 			n = n * 10 + (*cp++ - '0');
 		}
 	}
-	len = strlen(SD) + strlen(host) +8;
+	len = strlen(SD) + 15;
 	if ((cfname = malloc(len)) == NULL)
 		fatal("out of memory");
-	(void) snprintf(cfname, len, "%s/cfA%03d%s", SD, n, host);
+	(void) sprintf(cfname, "%s/cfA%03dAXXXXXX", SD, n);
+	if (mktemp(cfname) == NULL)
+		fatal("couldn't create unique control file");
 	if ((tfname = strdup(cfname)) == NULL || (dfname = strdup(cfname)) == NULL)
 		fatal("out of memory");
 	inchar = strlen(SD) + 3;
@@ -725,7 +724,7 @@ mktemps()
 	dfname[inchar-2] = 'd';
 	n = (n + 1) % 1000;
 	(void) lseek(fd, 0L, 0);
-	snprintf(buf, sizeof(buf), "%03d\n", n);
+	sprintf(buf, "%03d\n", n);
 	(void) write(fd, buf, strlen(buf));
 	(void) close(fd);	/* unlocks as well */
 }
@@ -733,7 +732,6 @@ mktemps()
 /*VARARGS1*/
 fatal(msg, a1, a2, a3)
 	char *msg;
-	void *a1, *a2, *a3;
 {
 	printf("%s: ", name);
 	printf(msg, a1, a2, a3);
